import pymysql


class DataBase:
    def __init__(self, user, password, host, port, database=None):
        self.database = database
        self.port = port
        self.host = host
        self.password = password
        self.user = user
        self.cursor = None
        self.connector = None

    def connection(self):
        self.connector = pymysql.connect(user=self.user, password=self.password, host=self.host, port=self.port,
                                         database=self.database)  # Connection à la base de donnée.
        self.cursor = self.connector.cursor()  # Création du curseur.

    def exec(self, body, args):  # Fonction permettant d'inserer des données dans la base de donnée
        self.connection()  # Connection avec la base de données.
        self.cursor.execute(body, args)  # Execution de la query.
        self.cursor.close()  # Fermeture du curseur.
        self.connector.commit()  # Enregistrement dans la base de donnée.
        self.connector.close()  # Fermeture de la connexion.

    def select(self, body, args, number_of_data=None):  # Fonction permettant d'obtenir des données de la base de donnée
        self.connection()  # Connection avec la base de données.
        self.cursor.execute(body, args)  # Execution de la query.
        self.cursor.close()  # Fermeture du curseur.
        self.connector.close()  # Fermeture de la connexion.

        if not number_of_data:  # Si aucune valeur n'est défini, alors on prends toute les valeurs
            data = self.cursor.fetchall()
        elif number_of_data == 1:  # Si la variable est sur 1, alors on prends qu'une seul valeur
            data = self.cursor.fetchone()
        else:  # Si la variable est autre chose que 1 ou undefined, alors on renvoie le nombre de valeurs demandé.
            data = self.cursor.fetchmany(number_of_data)

        return data



from smtplib import SMTP_SSL
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from flask import request
from random import randint


def send_verification_email(database):
    conn_url = database.select('''SELECT content FROM cantina_administration.config WHERE name='SMTP_URL' ''', None,
                               number_of_data=1)[0]
    conn_port = database.select('''SELECT content FROM cantina_administration.config WHERE name='SMTP_PORT' ''', None,
                                number_of_data=1)[0]
    conn_email = database.select('''SELECT content FROM cantina_administration.config WHERE name='SMTP_EMAIL' ''', None,
                                 number_of_data=1)[0]
    conn_passwd = database.select('''SELECT content FROM cantina_administration.config WHERE name='SMTP_PASSWD' ''',
                                  None, number_of_data=1)[0]

    subject = database.select('''SELECT content FROM cantina_administration.config WHERE name='MAIL_VERIF_SUBJECT' ''',
                              None, number_of_data=1)[0]
    message = database.select('''SELECT content FROM cantina_administration.config WHERE name='MAIL_VERIF_CONTENT' ''',
                              None, number_of_data=1)[0]
    destinataire = database.select('''SELECT email FROM cantina_administration.user WHERE token = %s''',
                                   (request.cookies.get('token')), number_of_data=1)[0]

    db_code = database.select('''SELECT email_verification_code FROM cantina_administration.user WHERE token = %s''',
                              (request.cookies.get('token')), number_of_data=1)[0]

    if db_code is None or db_code == 'reset':
        code = get_rand_num_for_email_verif()
        database.exec("""UPDATE cantina_administration.user SET email_verification_code = %s WHERE token = %s""",
                      (code, request.cookies.get('token')))
    else:
        code = database.select('''SELECT email_verification_code FROM cantina_administration.user WHERE token = %s''',
                               (request.cookies.get('token')), number_of_data=1)[0]

    if conn_url is None or conn_port is None or conn_email is None:
        raise 'error1: Configuration incomplète ou inexistante.'

    if subject is None or message is None:
        return 'error2: Message prédéfinis incomplet ou inexistant'

    mail = MIMEMultipart()
    mail['From'] = conn_email
    mail['To'] = destinataire
    mail['Subject'] = subject

    content = MIMEText(message.format(code).encode('utf-8'), 'plain', 'utf-8')
    mail.attach(content)

    smtp_server = SMTP_SSL(conn_url, int(conn_port))
    smtp_server.ehlo()
    smtp_server.login(conn_email, conn_passwd)

    smtp_server.sendmail(conn_email, destinataire, mail.as_string())
    smtp_server.close()


def get_rand_num_for_email_verif():
    code = ''
    for i in range(0, 6):
        code = code + str(randint(0, 9))

    return code



from flask import request
from pyotp import totp
from werkzeug.exceptions import BadRequestKeyError


def verify_login(database):
    token = request.cookies.get('token')
    token_validation = database.select("""SELECT id FROM cantina_administration.user WHERE token=%s""", (token,),
                                       number_of_data=1)
    validation = request.cookies.get('validation')
    validation_from_db = database.select("""SELECT content FROM cantina_administration.config WHERE name=%s""",
                                         ("secret_token",), number_of_data=1)

    return True if token_validation is not None and validation == validation_from_db[0] else False


def verify_A2F(database):
    try:
        key = totp.TOTP(database.select('''SELECT A2F_secret FROM cantina_administration.user WHERE username=%s''',
                                        (request.form['username']), number_of_data=1)[0])
    except BadRequestKeyError:
        key = totp.TOTP(database.select('''SELECT A2F_secret FROM cantina_administration.user WHERE token=%s''',
                                        (request.cookies.get('token')), number_of_data=1)[0])
    return key.verify(request.form['a2f-code'].replace(" ", ""))